# -*- coding: utf-8 -*-

"""
functions to download and clean SHS dataset
"""

import os
import re
import urllib.request
import logging
import logging.config

from shsdl.utils import setup_logging

log = logging.getLogger(__name__)
setup_logging()


def get_dataset_url(academic_type):
    """get the url for retrieve the dataset file

    :param academic_type: type of the dataset, "train" or "test"
    :return: url
    """

    log.info("retrieve url for SHS {} dataset".format(academic_type))
    return "https://raw.githubusercontent.com/" + \
           "tbertinmahieux/MSongsDB/master/Tasks_Demos/CoverSongs/" + \
           "shs_dataset_{}.txt".format(academic_type)


def load_data_shsid(academic_type):
    """load the dataset of the specified type.

    The returned data is a list of tuples, where each tuple is in format (work_id_list, performance_id_list).
    We have multiple work_ids for highly similar works.
    The returned data is immutable.

    :param academic_type: type of the dataset, "train" or "test"
    :return: dataset
    """
    log.info("start loading SHS {} dataset".format(academic_type))
    url = get_dataset_url(academic_type)
    data = urllib.request.urlopen(url).read().decode("utf-8").split("\n")
    cleaned_data = [d for d in data if len(d) > 0 and d[0] != "#"]
    work_start = [i for i in range(len(cleaned_data)) if cleaned_data[i][0] == "%"]
    work_start.append(len(cleaned_data))

    formatted_data = list()
    for i in range(len(work_start)-1):
        current_start = work_start[i]
        current_end = work_start[i+1] - 1
        # only match positive work_ids
        work_id_list = [int(v) for v in cleaned_data[current_start][1:].split(",") if re.match("^[0-9]+$", v)]
        if len(work_id_list) > 0:
            performance_id_list = list()
            for j in range(current_start+1, current_end+1):
                performance_id = int(cleaned_data[j].split("<SEP>")[-1])
                if performance_id > 0:
                    performance_id_list.append(performance_id)
            formatted_data.append((tuple(work_id_list), tuple(performance_id_list)))
            log.debug("match works {} with performances {}".format(work_id_list, performance_id_list))
        else:
            log.debug("no valid work id found on line: {}".format(cleaned_data[current_start]))
    log.info("loaded SHS {} dataset".format(academic_type))
    return formatted_data

def load_data_msdid(academic_type):
    """load the dataset of the specified type.

    The returned data is a list of tuples, where each tuple is in format (work_id_list, performance_id_list).
    We have multiple work_ids for highly similar works.
    The returned data is immutable.

    :param academic_type: type of the dataset, "train" or "test"
    :return: dataset
    """
    log.info("start loading SHS {} dataset".format(academic_type))
    url = get_dataset_url(academic_type)
    data = urllib.request.urlopen(url).read().decode("utf-8").split("\n")
    cleaned_data = [d for d in data if len(d) > 0 and d[0] != "#"]
    work_start = [i for i in range(len(cleaned_data)) if cleaned_data[i][0] == "%"]
    work_start.append(len(cleaned_data))

    formatted_data = list()
    for i in range(len(work_start)-1):
        current_start = work_start[i]
        current_end = work_start[i+1] - 1

        msd_id_list = list()
        for j in range(current_start+1, current_end+1):
            msd_id = cleaned_data[j].split("<SEP>")[0]
            msd_id_list.append(msd_id)
        formatted_data.append((i, tuple(msd_id_list)))
        log.debug("class {}: get MSD performances {}".format(i, msd_id_list))

    log.info("loaded SHS {} dataset".format(academic_type))
    return formatted_data


def get_pairs_format(dataset_list, np_ratio=1):

    from itertools import combinations
    from numpy.random import choice

    hash_table = dict()
    all_tracks = list()

    combo_x1 = list()
    combo_x2 = list()
    combo_label = list()

    for label, track_ids in dataset_list:

        combo = combinations(track_ids, 2)
        for pair_x1, pair_x2 in combo:
            combo_x1.append(pair_x1)
            combo_x2.append(pair_x2)
            combo_label.append(1)

        for track in track_ids:
            hash_table[track] = label
            all_tracks.append(track)

    total_true_labels = len(combo_label)
    negatives = total_true_labels * np_ratio

    count = 0

    while count < negatives:
        pair_x1, pair_x2 = choice(all_tracks, 2, replace=False)
        if hash_table[pair_x1] != hash_table[pair_x2]:
            combo_x1.append(pair_x1)
            combo_x2.append(pair_x2)
            combo_label.append(0)
            count += 1

    return combo_x1, combo_x2, combo_label











