# -*- coding: utf-8 -*-

"""classes and functions for scraping performances/cases
"""

import re
from bs4 import BeautifulSoup
import urllib.request
import logging
import logging.config

from shsdl.utils import setup_logging
from shsdl.schema import PFM_ID, PFM_TYPE, PFM_ACAD_TYPE, YTB_ID, WORK_ID

log = logging.getLogger(__name__)
setup_logging()


def initialize_performance(url_path, work_id=None, get_youtube=True):
    """initialize performance based on the url path

    :param url_path: relative path to the performance
    :param work_id: work id of the performance
    :param get_youtube: if need to get youtube link
    :return: Performance object
    """

    if re.match(Performance.url_path_regex, url_path):
        performance_id = int(url_path.split("/")[-2])
        return Performance(performance_id, work_id, get_youtube)

    elif re.match(Case.url_path_regex, url_path):
        performance_id = int(url_path.split("/")[-1])
        return Case(performance_id, work_id, get_youtube)

    else:
        raise Exception("failed to match url path {}".format(url_path))


class AbstractPerformance:
    """The abstract class to represent a performance or a case
    """

    url_path_regex = ""
    url_template = ""
    performance_type = ""

    def __init__(self, performance_id, work_id=None, get_youtube=True):
        """

        The initializer takes a performance id, and has the option to grab the youtube id.
        work_id is optional because we can directly scrape a performance without knowing its work
        get_youtube is optional because we can know beforehand if the performance is in youtube

        :param performance_id: int performance id
        :param work_id: int work id
        :param get_youtube: if we need to get the youtube id for the performance
        """
        self.performance_id = performance_id
        self.db_id = "{}-{}".format(self.performance_type, performance_id)
        self.url = self.url_template.format(self.performance_id)
        self.work_id = work_id
        if get_youtube:
            self.youtube_id = self.get_youtube_id()
        else:
            log.debug("indicated no youtube link, skip getting youtube id")
            self.youtube_id = None

        log.info("initialized {}".format(self))

    def __str__(self):
        raise Exception("Abstract class for Performance and Case, shouldn't be constructed directly")

    def get_content_soup(self):
        """get the content soup object of the performance

        :return: BeautifulSoup object with performance content
        """
        log.debug("get content soup for {}".format(self))
        contents = urllib.request.urlopen(self.url).read()
        soup = BeautifulSoup(contents, 'html.parser')
        return soup

    def get_youtube_id(self):
        """get the youtube id of the performance
        :return: youtube id string
        """
        raise Exception("Abstract class for Performance and Case, shouldn't call this method directly")

    def get_db_object(self, academic_type):
        """get object that can be posted to a noSQL database

        :param academic_type: type of the dataset, "train" or "test"
        :return: the json formatted object
        """
        basic = {
            PFM_ID: self.db_id,
            PFM_TYPE: self.performance_type,
            PFM_ACAD_TYPE: academic_type
        }
        if self.work_id:
            basic[WORK_ID] = self.work_id
        if self.youtube_id:
            basic[YTB_ID] = self.youtube_id
        return basic


class Performance(AbstractPerformance):
    """the class the represents the performances in secondhandsongs.com
    """

    url_path_regex = "/performance/[0-9]+/versions"
    url_template = "https://secondhandsongs.com/performance/{}/versions"
    performance_type = "performance"

    def __str__(self):
        return "<Performance {}>".format(self.performance_id)

    def get_youtube_id(self):
        log.debug("try getting youtube id for {}".format(self))
        soup = self.get_content_soup()
        video = soup.find("div", "perf_video")
        if video:
            youtube_id = video.iframe['src'].split("//www.youtube.com/embed/")[1].split("?rel=0")[0]
            log.debug("found youtube id: {}".format(youtube_id))
            return youtube_id
        else:
            log.debug("cannot find youtube id for {}".format(self))
            return None


class Case(AbstractPerformance):
    """the class the represents the cases in secondhandsongs.com
    """

    url_path_regex = "/case/[0-9]+"
    url_template = "https://secondhandsongs.com/case/{}"
    performance_type = "case"

    def __str__(self):
        return "<Performance-Case {}>".format(self.performance_id)

    def get_youtube_id(self):
        log.debug("try getting youtube id for {}".format(self))
        soup = self.get_content_soup()
        iframe = soup.find('iframe')
        if iframe:
            youtube_id = iframe['src'].split("//www.youtube.com/embed/")[1].split("?rel=0")[0]
            log.debug("found youtube id: {}".format(youtube_id))
            return youtube_id
        else:
            log.debug("cannot find youtube id for {}".format(self))
            return None

