# -*- coding: utf-8 -*-

"""classes and functions for scraping works
"""

from bs4 import BeautifulSoup
import urllib.request
import logging
import logging.config
import re

from shsdl.utils import setup_logging
from shsdl.performance import initialize_performance
from shsdl.schema import WORK_ID, WORK_ID_DUP, WORK_PFM_LIST

log = logging.getLogger(__name__)
setup_logging()

WORK_URL_FORMAT = 'https://secondhandsongs.com/work/{}/versions#nav-entity'


class Work:
    """the class the represents the works in secondhandsongs.com
    """

    def __init__(self, work_id):
        """

        The initializer takes a work id, and finds duplicated work ids and performances associated

        :param work_id: int work id
        """

        self.work_id = work_id
        self.duplicates = list()
        self.url = WORK_URL_FORMAT.format(work_id)
        self.performances = self.get_performance_list()
        log.info("initialized {}".format(self))

    def __str__(self):
        return "<WORK {}>".format(self.work_id)

    def __repr__(self):
        return self.__str__()

    def get_performance_list(self):

        log.info("get performance list for {}".format(self))

        contents = urllib.request.urlopen(self.url).read()
        soup = BeautifulSoup(contents, 'html.parser')
        tables = soup.find_all("table", "table data object table-striped")

        total_useful, total = 0, 0
        performances = list()
        for table in tables:

            if "versions" in table['id']:
                version_number = table['id'].split("versions")[-1]
                if re.match("^[0-9]+$", version_number):
                    table_id = int(version_number)
                    if table_id != self.work_id and table_id not in self.duplicates:
                        self.duplicates.append(table_id)
            rows = table.find_all("tr")

            for row in rows:
                row_entries = row.find_all("td")
                url_path = row_entries[1].a['href']
                total += 1

                # if it has the icon for youtube, the cover is considered useful
                if row_entries[0].i:
                    total_useful += 1
                    performances.append(initialize_performance(url_path, self.work_id, True))
                else:
                    performances.append(initialize_performance(url_path, self.work_id, False))

        if len(performances) == 0:
            log.debug("no performance scraped for {}".format(self))
        else:
            log.debug("performance scraped for {}, {} / {} has youtube link".format(
                self.work_id, total_useful, total
            ))

        return performances

    def get_db_object(self):
        """get object that can be posted to a noSQL database

        :return: the json formatted object
        """
        return {
            WORK_ID: self.work_id,
            WORK_ID_DUP: self.duplicates,
            WORK_PFM_LIST: [pfm.db_id for pfm in self.performances]
        }
